#ifndef  MODULES_PARTITION_H
#define  MODULES_PARTITION_H

/*
**   partition.h
**
**   (C) Copyright 2004 Rocklyte Systems.
**       All Rights Reserved.
*/

#ifndef  MAIN_H
#include <pandora/main.h>
#endif

#define MODVERSION_PARTITION ((FLOAT)1.0)

struct Partition;

struct PartitionDisk {
   STRING Device;                 /* Device path */
   ULONG  Size;                   /* Size of the disk in sectors */
   STRING Model;                  /* Drive model */
   struct Partition *Partitions;  /* First partition in the list */
   struct PartitionDisk *Next;    /* Next available disk device */
   ULONG  Heads;                  /* Drive geometry - heads */
   ULONG  Cylinders;              /* Drive geometry - cylinders */
   ULONG  Sectors;                /* Drive geometry - sectors per track */
   UBYTE  MBR[512];               /* The complete master boot record (first sector) */
};

struct PartitionCHS {
   UBYTE Head;
   UBYTE Sector;
   UBYTE Cylinder;
};

struct Partition {
   struct PartitionDisk *Disk;  /* Reference to the parent disk structure */
   struct Partition *Next;      /* Next partition in the table */
   STRING Device;               /* Partition device name */
   STRING FileSystem;           /* Partition file system type */
   WORD   Number;               /* Sequential identifier for the partiton */
   WORD   Flags;                /* PTYPE_EXTENDED, PTYPE_LOGICAL ... */
   LONG   FSType;               /* FSTYPE */
   ULONG  Sector;               /* Starting position on the disk, in 512 byte sectors */
   UBYTE  Label[32];            /* File system volume label */
   ULONG  EndSector;            /* Sector at which the partition ends */
   ULONG  Offset;               /* For logical partitions - indicates where the partition actually starts (usually 1 cylinder away) */
   /*** Private ***/
   UBYTE  Code;                 /* The partition filesystem code */
   struct PartitionCHS Start;
   struct PartitionCHS End;
};

enum {
   FSTYPE_UNKNOWN=0,  /* Partition has an unknown file system, or is unformatted */
   FSTYPE_EXT2,
   FSTYPE_EXT3,
   FSTYPE_SWAP,
   FSTYPE_REISERFS,
   FSTYPE_JFS,
   FSTYPE_XFS,
   FSTYPE_FAT32,
   FSTYPE_FAT16,
   FSTYPE_NTFS,
   FSTYPE_HFS,
   FSTYPE_UFS_SUN,
   FSTYPE_UFS_HP,
   FSTYPE_EXT_PARTITION, /* Extended partition with no file system */
   FSTYPE_FREESPACE      /* Not a real partition at all - just free space */
};

#define PTYPE_FREESPACE 0x0001
#define PTYPE_EXTENDED  0x0002
#define PTYPE_LOGICAL   0x0004
#define PTYPE_PRIMARY   0x0010
#define PTYPE_ACTIVE    0x0020 /* Partition is active/bootable */

#define PCODE_EMPTY		0x00
#define PCODE_DOS12		0x01
#define PCODE_DOS16S		0x04
#define PCODE_DOSEXT		0x05
#define PCODE_DOS16L		0x06
#define PCODE_NTFS		0x07
#define PCODE_OS2BOOT           0x0a
#define PCODE_WFAT32		0x0b
#define PCODE_WFAT32L		0x0c
#define PCODE_WFAT16L           0x0e
#define PCODE_WEXT		0x0f
#define PCODE_DOS12_HIDE        0x11
#define PCODE_DOS16S_HIDE       0x14
#define PCODE_DOS16L_HIDE       0x16
#define PCODE_NTFS_HIDE         0x17
#define PCODE_WFAT32_HIDE       0x1b
#define PCODE_WFAT32L_HIDE      0x1c
#define PCODE_WFAT16_HIDE       0x1e
#define PCODE_MINIX_DR		0x41
#define PCODE_SWAP_DR		0x42
#define PCODE_LINUX_DR		0x43
#define PCODE_MINIX		0x81
#define PCODE_LSWAP		0x82
#define PCODE_LINUX		0x83
#define PCODE_LINUXEXT		0x85
#define PCODE_NOFILESYSTEM      0xda

#define PCODE_UNKNOWN		0x100

/****************************************************************************/

#ifndef LOCAL_PARTITION

struct PartitionBase {
   struct PartitionDisk * (*GetPartitionDisks)(void);
   void   (*RescanPartitions)(void);
   ERROR  (*WritePartitionTable)(struct PartitionDisk *);
   ERROR  (*DeletePartition)(struct Partition *);
   ERROR  (*CreatePartition)(struct Partition *, ULONG, LONG);
   STRING (*TranslatePartitionFS)(LONG);
   LONG   (*RebootPartitions)(void);
   STRING (*TranslatePartitionCode)(LONG);
};

#define CreatePartition(a,b,c)    (PartitionBase->CreatePartition(a,b,c))
#define DeletePartition(a)        (PartitionBase->DeletePartition(a))
#define GetPartitionDisks()       (PartitionBase->GetPartitionDisks())
#define RescanPartitions()        (PartitionBase->RescanPartitions())
#define RebootPartitions()        (PartitionBase->RebootPartitions())
#define TranslatePartitionCode(a) (PartitionBase->TranslatePartitionCode(a))
#define TranslatePartitionFS(a)   (PartitionBase->TranslatePartitionFS(a))
#define WritePartitionTable(a)    (PartitionBase->WritePartitionTable(a))

#else

static ERROR  CreatePartition(struct Partition *, ULONG, LONG);
static ERROR  DeletePartition(struct Partition *);
static struct PartitionDisk * GetPartitionDisks(void);
static void   RescanPartitions(void);
static LONG   RebootPartitions(void);
static STRING TranslatePartitionCode(LONG);
static STRING TranslatePartitionFS(LONG);
static ERROR  WritePartitionTable(struct PartitionDisk *);

#endif /* LOCAL_PARTITION */

#endif /* MODULES_PARTITION_H */
